@@ -38,7 +38,8 @@ public:
 	virtual FiStatus Setup(bool bDynType);
 	
 	virtual int GetFileFd();
-	uint64_t GetTransferCount();
+	uint64_t TakeTransferCount();
+	uint64_t GetTransferCountUnlocked() { return m_nIncommingCount; }
 	// send helper like wrapper for sendfile. Just declare virtual here to make it better customizable later.
 	virtual ssize_t SendData(int confd, int filefd, off_t &nSendPos, size_t nMax2SendNow)=0;
 	
@@ -47,6 +48,13 @@ public:
 	virtual bool DownloadStartedStoreHeader(const header & h, size_t hDataLen,
 			const char *pNextData,
 			bool bForcedRestart, bool &bDoCleanRestart) =0;
+	/**
+	 * For special implementations, member the original header data.
+	 * By default, drop that data since we pickup everything important already.
+	 */
+	virtual void SetRawResponseHeader(std::string) {}
+	virtual const std::string& GetRawResponseHeader() { return sEmptyString; }
+
 	void IncDlRefCount();
 	void DecDlRefCount(const mstring & sReasonStatusLine);
 	//virtual void SetFailureMode(const mstring & message, FiStatus fist=FIST_ERROR,
@@ -70,6 +78,8 @@ public:
 	// returns when the state changes to complete or error
 	FiStatus WaitForFinish(int *httpCode=nullptr);
 
+	FiStatus WaitForFinish(int *httpCode, unsigned check_interval, const std::function<void()> &check_func);
+
 	bool SetupClean(bool bForce=false);
 	
 	/// mark the item as complete as-is, assuming that sizeseen is correct
@@ -156,11 +166,15 @@ public:
 	inline fileItemMgmt() {}
 	inline tFileItemPtr getFiPtr() {return m_ptr;}
 	inline operator bool() const {return (bool) m_ptr;}
-
-	tFileItemPtr m_ptr;
+	/**
+	 * Unregister from global lookup (running cleanup actions as needed)
+	 * and clear internal reference.
+	 */
+	void reset();
 
 private:
-	void Unreg();
+
+	tFileItemPtr m_ptr;
 
 	fileItemMgmt(const fileItemMgmt &src);
 	fileItemMgmt& operator=(const fileItemMgmt &src);
