@@ -50,8 +50,11 @@ tSpecOpDetachable::~tSpecOpDetachable()
 cmstring GetFooter()
 {
         return mstring("<hr><address>Server: ") + cfg::agentname
-                + "&nbsp;&nbsp;|&nbsp;&nbsp;<a\nhref=\"https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=QDCK9C2ZGUKZY&source=url\">Donate!"
-                "</a>&nbsp;&nbsp;|&nbsp;&nbsp;<a\nhref=\"http://www.unix-ag.uni-kl.de/~bloch/acng/\">Apt-Cacher NG homepage</a></address>";
+                + "&nbsp;&nbsp;"
+				"|&nbsp;&nbsp;<a\nhref=\"/\">Usage Information</a>&nbsp;&nbsp;"
+				"|&nbsp;&nbsp;<a\nhref=\"https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=QDCK9C2ZGUKZY&source=url\">Donate!"
+                "</a>&nbsp;&nbsp;"
+				"|&nbsp;&nbsp;<a\nhref=\"http://www.unix-ag.uni-kl.de/~bloch/acng/\">Apt-Cacher NG homepage</a></address>";
 }
 
 std::string to_base36(unsigned int val)
@@ -176,15 +179,31 @@ void tSpecOpDetachable::Run()
 			while(true)
 			{
 				int r = sendbuf.sysread(m_logFd);
-				if(r < 0)
+				if(r < 0) // error
 					goto finish_action;
+				if(r == -EAGAIN)
+				{
+					g_StateCv.wait_for(g, 1, 1);
+					if(!nBgTimestamp)
+						break;
+					continue;
+				}
 				if(r == 0)
 				{
-					g_StateCv.wait_for(g, 10, 1);
-					break;
+					g_StateCv.wait_for(g, 5, 1);
+					if(!nBgTimestamp)
+						break;
+					continue;
 				}
 				SendChunkRemoteOnly(sendbuf.rptr(), sendbuf.size());
 				sendbuf.clear();
+				if(r>0)
+				{
+					// read more once?
+					continue;
+				}
+				if(!nBgTimestamp)
+					break;
 			}
 		}
 		// unreachable
