@@ -32,7 +32,7 @@ conn::conn(unique_fd fd, const char *c) :
 	if(c) // if nullptr, pick up later when sent by the wrapper
 		m_sClientHost=c;
 
-	LOGSTART2("con::con", "fd: " << m_confd << ", clienthost: " << c);
+	LOGSTARTx("con::con", m_confd, c);
 
 #ifdef DEBUG
 	m_nProcessedJobs=0;
@@ -223,9 +223,11 @@ void conn::WorkLoop() {
 
 		if(ready == 0)
 		{
-			USRDBG("Timeout occurred, apt client disappeared silently?");
 			if(GetTime() > client_timeout)
+			{
+				USRDBG("Timeout occurred, apt client disappeared silently?");
 				return; // yeah, time to leave
+			}
 			continue;
 		}
 		else if (ready<0)
