@@ -30,10 +30,16 @@ using namespace std;
 namespace acng
 {
 
-static cmstring sGenericError("567 Unknown download error occured");
+static cmstring sGenericError("502 Bad Gateway");
 
-// those are not allowed to be forwarded
-static const auto taboo =
+// those are not allowed to be forwarded ever
+static const auto tabooHeadersForCaching =
+{
+	string("Host"), string("Cache-Control"),
+	string("Proxy-Authorization"), string("Accept"),
+	string("User-Agent"), string("Accept-Encoding")
+};
+static const auto tabooHeadersPassThrough =
 {
 	string("Host"), string("Cache-Control"),
 	string("Proxy-Authorization"), string("Accept"),
@@ -122,7 +128,8 @@ struct tDlJob
 	const tHttpUrl *m_pCurBackend=nullptr;
 
 	uint_fast8_t m_eReconnectASAP =0;
-	bool m_bBackendMode=false;
+	bool m_bBackendMode = false;
+	bool m_isPassThroughRequest = false;
 
 	off_t m_nRest =0;
 
@@ -134,10 +141,11 @@ struct tDlJob
 			const tHttpUrl *pUri,
 			const cfg::tRepoData * pRepoData,
 			const std::string *psPath,
-			int redirmax) :
+			int redirmax, bool isPassThroughRequest) :
 			m_pStorage(pFi),
 			m_parent(*p),
 			m_pRepoDesc(pRepoData),
+			m_isPassThroughRequest(isPassThroughRequest),
 			m_nRedirRemaining(redirmax)
 	{
 		LOGSTART("tDlJob::tDlJob");
@@ -159,8 +167,15 @@ struct tDlJob
 	~tDlJob()
 	{
 		LOGSTART("tDlJob::~tDlJob");
+		const auto* pError = &sErrorMsg;
+		if(sErrorMsg.empty())
+		{
+			pError = &sGenericError;
+			// XXX: BS? Leaving without error is not a bad situation
+			// log::err(RemoteUri(true)  + " -- bad download descriptor, exited without leaving error message");
+		}
 		if (m_pStorage)
-			m_pStorage->DecDlRefCount(sErrorMsg.empty() ? sGenericError : sErrorMsg);
+			m_pStorage->DecDlRefCount(*pError);
 	}
 
 	inline void ExtractCustomHeaders(LPCSTR reqHead)
@@ -168,18 +183,24 @@ struct tDlJob
 		if(!reqHead)
 			return;
 		header h;
-		bool forbidden=false;
+		// continuation of header line
+		bool forbidden = false;
 		h.Load(reqHead, (unsigned) std::numeric_limits<int>::max(),
-				[this, &forbidden](cmstring& key, cmstring& rest)
+				[this, &forbidden](cmstring &key, cmstring &rest)
 				{
-			// heh, continuation of ignored stuff or without start?
-			if(key.empty() && (m_extraHeaders.empty() || forbidden))
-				return;
-			forbidden = taboo.end() != std::find_if(taboo.begin(), taboo.end(),
-					[&key](cmstring &x){return scaseequals(x,key);});
-			if(!forbidden)
-				m_extraHeaders += key + rest;
-				}
+					// heh, continuation of ignored stuff or without start?
+						if(key.empty() && (m_extraHeaders.empty() || forbidden))
+						{
+							return;
+						}
+						const auto& taboo = m_isPassThroughRequest ? tabooHeadersPassThrough : tabooHeadersForCaching;
+
+						forbidden = taboo.end() != std::find_if(taboo.begin(), taboo.end(),
+								[&](cmstring &x)
+								{	return scaseequals(x,key);});
+						if(!forbidden)
+							m_extraHeaders += key + rest;
+					}
 		);
 	}
 
@@ -212,7 +233,6 @@ struct tDlJob
 		m_pCurBackend = nullptr;
 		bool bWasBeMode = m_bBackendMode;
 		m_bBackendMode = false;
-		sErrorMsg = "500 Bad redirection (path)";
 
 		auto sLocationDecoded = UrlUnescape(pNewUrl);
 
@@ -235,7 +255,10 @@ struct tDlJob
 		if(bWasBeMode)
 		{
 			if(!m_pCurBackend)
+			{
+				sErrorMsg = "500 Bad redirection (path)";
 				return false;
+			}
 			auto sPathBackup=m_remoteUri.sPath;
 			m_remoteUri=*m_pCurBackend;
 			m_remoteUri.sPath+=sPathBackup;
@@ -266,7 +289,7 @@ struct tDlJob
 				const auto bliter = blacklist.find(
 						make_pair(m_pCurBackend->sHost, m_pCurBackend->GetPort()));
 				if (bliter == blacklist.end())
-					return true;
+					LOGRET(true);
 			}
 
 			// look in the constant list, either it's usable or it was blacklisted before
@@ -276,23 +299,29 @@ struct tDlJob
 				if (bliter == blacklist.end())
 				{
 					m_pCurBackend = &bend;
-					return true;
+					LOGRET(true);
 				}
 
 				// uh, blacklisted, remember the last reason
-				sReasonMsg = bliter->second;
+				if(sReasonMsg.empty())
+				{
+					sReasonMsg = bliter->second;
+					LOG(sReasonMsg);
+				}
 			}
-			return false;
+			if(sReasonMsg.empty())
+				sReasonMsg = "502 Mirror blocked due to repeated errors";
+			LOGRET(false);
 		}
 
 		// ok, not backend mode. Check the mirror data (vs. blacklist)
 		auto bliter = blacklist.find(
 				make_pair(GetPeerHost().sHost, GetPeerHost().GetPort()));
 		if (bliter == blacklist.end())
-			return true;
+			LOGRET(true);
 
 		sReasonMsg = bliter->second;
-		return false;
+		LOGRET(false);
 	}
 
 	// needs connectedHost, blacklist, output buffer from the parent, proxy mode?
@@ -402,18 +431,21 @@ struct tDlJob
 		}
 
 		if (m_pStorage->m_bCheckFreshness)
-			head << "Cache-Control: no-store,no-cache,max-age=0\r\n";
+			head << "Cache-Control: " /*no-store,no-cache,*/ "max-age=0\r\n";
 
 		if (cfg::exporigin && !xff.empty())
 			head << "X-Forwarded-For: " << xff << "\r\n";
 
 		head << cfg::requestapx
 				<< m_extraHeaders
-				<< "Accept: application/octet-stream\r\n"
-				"Accept-Encoding: identity\r\n"
-				"Connection: "
-				<< (cfg::persistoutgoing ? "keep-alive\r\n\r\n" : "close\r\n\r\n");
+				<< "Accept: application/octet-stream\r\n";
+		if(!m_isPassThroughRequest)
+		{
+			head << "Accept-Encoding: identity\r\n"
+					"Connection: " << (cfg::persistoutgoing ? "keep-alive\r\n" : "close\r\n");
 
+		}
+		head << "\r\n";
 #ifdef SPAM
 		//head.syswrite(2);
 #endif
@@ -494,7 +526,12 @@ struct tDlJob
 				}
 
 				ldbg("contents: " << std::string(inBuf.rptr(), hDataLen));
-				inBuf.drop((unsigned long) hDataLen);
+
+				if(m_pStorage)
+					m_pStorage->SetRawResponseHeader(string(inBuf.rptr(), hDataLen));
+
+				inBuf.drop(size_t(hDataLen));
+
 				if (h.type != header::ANSWER)
 				{
 					dbgline;
@@ -609,7 +646,7 @@ struct tDlJob
 					}
 				}
 
-				if(!m_pStorage->DownloadStartedStoreHeader(h, hDataLen,
+				if(!m_pStorage->DownloadStartedStoreHeader(h, size_t(hDataLen),
 						inBuf.rptr(), bHotItem, bDoRetry))
 				{
 					if(bDoRetry)
@@ -753,7 +790,7 @@ inline void dlcon::awaken_check()
 bool dlcon::AddJob(tFileItemPtr m_pItem, const tHttpUrl *pForcedUrl,
 		const cfg::tRepoData *pBackends,
 		cmstring *sPatSuffix, LPCSTR reqHead,
-		int nMaxRedirection)
+		int nMaxRedirection, bool isPassThroughRequest)
 {
 	if(!pForcedUrl)
 	{
@@ -773,7 +810,8 @@ bool dlcon::AddJob(tFileItemPtr m_pItem, const tHttpUrl *pForcedUrl,
 	LOGSTART2("dlcon::EnqJob", todo->m_remoteUri.ToURI(false));
 */
 	m_qNewjobs.emplace_back(
-			make_shared<tDlJob>(this, m_pItem, pForcedUrl, pBackends, sPatSuffix,nMaxRedirection));
+			make_shared<tDlJob>(this, m_pItem, pForcedUrl, pBackends, sPatSuffix,
+					nMaxRedirection,isPassThroughRequest));
 
 	m_qNewjobs.back()->ExtractCustomHeaders(reqHead);
 
@@ -806,8 +844,8 @@ dlcon::~dlcon()
 
 inline unsigned dlcon::ExchangeData(mstring &sErrorMsg, tDlStreamHandle &con, tDljQueue &inpipe)
 {
-	LOGSTART2("dlcon::ExchangeData",
-			"qsize: " << inpipe.size() << ", sendbuf size: "
+	LOGSTARTFUNC;
+	LOG("qsize: " << inpipe.size() << ", sendbuf size: "
 			<< m_sendBuf.size() << ", inbuf size: " << m_inBuf.size());
 
 	fd_set rfds, wfds;
@@ -859,21 +897,20 @@ inline unsigned dlcon::ExchangeData(mstring &sErrorMsg, tDlStreamHandle &con, tD
 
 		r=select(nMaxFd + 1, &rfds, &wfds, nullptr, CTimeVal().ForNetTimeout());
 		ldbg("returned: " << r << ", errno: " << errno);
-		if(m_bStopASAP || evabase::in_shutdown)
 		{
-			return HINT_DISCON;
+			setLockGuard
+			if (m_bStopASAP || evabase::in_shutdown)
+				return HINT_DISCON;
 		}
 
 		if (r == -1)
 		{
 			if (EINTR == errno)
 				continue;
-#ifdef MINIBUILD
-			string fer("select failed");
-#else
+			if(EBADF == errno) // that some times happen for no obvious reason
+				return HINT_DISCON;
 			tErrnoFmter fer("FAILURE: select, ");
 			LOG(fer);
-#endif
 			sErrorMsg = string("500 Internal malfunction, ") + fer;
 			return HINT_DISCON|EFLAG_JOB_BROKEN|EFLAG_MIRROR_BROKEN;
 		}
@@ -1150,8 +1187,7 @@ void dlcon::WorkLoop()
 
 	auto BlacklistMirror = [&](tDlJobPtr & job)
 	{
-		LOGSTART2("BlacklistMirror", "blacklisting " <<
-				job->GetPeerHost().ToURI(false));
+		LOGSTARTx("BlacklistMirror", job->GetPeerHost().ToURI(false));
 		m_blacklist[std::make_pair(job->GetPeerHost().sHost,
 				job->GetPeerHost().GetPort())] = sErrorMsg;
 	};
@@ -1362,11 +1398,13 @@ void dlcon::WorkLoop()
 
         // inner loop: plain communication until something happens. Maybe should use epoll here?
         loopRes=ExchangeData(sErrorMsg, con, inpipe);
-        ldbg("loopRes: "<< loopRes);
-        if(m_bStopASAP || evabase::in_shutdown)
-        	return;
-
-        /* check whether we have a pipeline stall. This may happen because a) we are done or
+		ldbg("loopRes: "<< loopRes);
+		{
+			setLockGuard
+			if (m_bStopASAP || evabase::in_shutdown)
+				return;
+		}
+		/* check whether we have a pipeline stall. This may happen because a) we are done or
          * b) because of the remote hostname change or c) the client stopped sending tasks.
          * Anyhow, that's a reason to put the connection back into the shared pool so either we
          * get it back from the pool in the next workloop cycle or someone else gets it and we
