@@ -101,7 +101,7 @@ void fileitem::DecDlRefCount(const string &sReason)
 	checkforceclose(m_filefd);
 }
 
-uint64_t fileitem::GetTransferCount()
+uint64_t fileitem::TakeTransferCount()
 {
 	setLockGuard;
 	uint64_t ret=m_nIncommingCount;
@@ -143,7 +143,7 @@ void fileitem::ResetCacheState()
 
 fileitem::FiStatus fileitem::Setup(bool bCheckFreshness)
 {
-	LOGSTART2("fileitem::Setup", bCheckFreshness);
+	LOGSTARTFUNCx(bCheckFreshness);
 
 	setLockGuard;
 
@@ -308,6 +308,19 @@ fileitem::FiStatus fileitem::WaitForFinish(int *httpCode)
 	return m_status;
 }
 
+fileitem::FiStatus fileitem::WaitForFinish(int *httpCode, unsigned check_interval, const std::function<void()> &check_func)
+{
+	lockuniq g(this);
+	while(m_status<FIST_COMPLETE)
+	{
+		if(wait_for(g, 1, 1)) // on timeout
+			check_func();
+	}
+	if(httpCode)
+		*httpCode=m_head.getStatus();
+	return m_status;
+}
+
 inline void _LogWithErrno(const char *msg, const string & sFile)
 {
 	tErrnoFmter f;
@@ -655,8 +668,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, size_t
 bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 {
 	setLockGuard;
-
-	LOGSTART2("fileitem::StoreFileData", "status: " <<  (int) m_status << ", size: " << size);
+	LOGSTARTFUNCx(int(size));
 
 	// something might care, most likely... also about BOUNCE action
 	notifyAll();
@@ -729,51 +741,55 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 fileItemMgmt::~fileItemMgmt()
 {
 	LOGSTART("fileItemMgmt::~fileItemMgmt");
-	Unreg();
+	reset();
 }
 
-inline void fileItemMgmt::Unreg()
+void fileItemMgmt::reset()
 {
-	LOGSTART("fileItemMgmt::Unreg");
+	LOGSTARTFUNC
 
 	if(!m_ptr) // unregistered before?
 		return;
 
 	lockguard managementLock(mapItemsMx);
 
-	// invalid or not globally registered?
-	if(m_ptr->m_globRef == mapItems.end())
-		return;
+	bool wasGloballyRegistered = m_ptr->m_globRef != mapItems.end();
 
 	auto local_ptr(m_ptr); // might disappear
 	lockguard fitemLock(*local_ptr);
 
 	if ( -- m_ptr->usercount <= 0)
 	{
+		m_ptr->notifyAll();
+
 		if(m_ptr->m_status < fileitem::FIST_COMPLETE && m_ptr->m_status != fileitem::FIST_INITED)
 		{
 			ldbg("usercount dropped to zero while downloading?: " << (int) m_ptr->m_status);
 		}
 
-		// some file items will be held ready for some time
-		time_t when(0);
-		if (MAXTEMPDELAY && m_ptr->m_bCheckFreshness &&
-				m_ptr->m_status == fileitem::FIST_COMPLETE &&
-				((when = m_ptr->m_nTimeDlStarted+MAXTEMPDELAY) > GetTime()))
+		if(wasGloballyRegistered)
 		{
-			cleaner::GetInstance().ScheduleFor(when, cleaner::TYPE_EXFILEITEM);
-			return;
-		}
+			// some file items will be held ready for some time
 
+			time_t when(0);
+			if (MAXTEMPDELAY && m_ptr->m_bCheckFreshness
+					&& m_ptr->m_status == fileitem::FIST_COMPLETE
+					&& ((when = m_ptr->m_nTimeDlStarted + MAXTEMPDELAY) > GetTime()))
+			{
+				cleaner::GetInstance().ScheduleFor(when, cleaner::TYPE_EXFILEITEM);
+				return;
+			}
+		}
 		// nothing, let's put the item into shutdown state
 		m_ptr->m_status = fileitem::FIST_DLSTOP;
 		m_ptr->m_head.frontLine="HTTP/1.1 500 Cache file item expired";
-		m_ptr->notifyAll();
-
-		LOG("*this is last entry, deleting dl/fi mapping");
-		mapItems.erase(m_ptr->m_globRef);
-		m_ptr->m_globRef = mapItems.end();
 
+		if (wasGloballyRegistered)
+		{
+			LOG("*this is last entry, deleting dl/fi mapping");
+			mapItems.erase(m_ptr->m_globRef);
+			m_ptr->m_globRef = mapItems.end();
+		}
 		// make sure it's not double-unregistered accidentally!
 		m_ptr.reset();
 	}
@@ -782,7 +798,7 @@ inline void fileItemMgmt::Unreg()
 
 bool fileItemMgmt::PrepareRegisteredFileItemWithStorage(cmstring &sPathUnescaped, bool bConsiderAltStore)
 {
-	LOGSTART2("fileitem::GetFileItem", sPathUnescaped);
+	LOGSTARTFUNCx(bConsiderAltStore);
 
 	try
 	{
@@ -841,12 +857,12 @@ bool fileItemMgmt::PrepareRegisteredFileItemWithStorage(cmstring &sPathUnescaped
 // make the fileitem globally accessible
 bool fileItemMgmt::RegisterFileItem(tFileItemPtr spCustomFileItem)
 {
-	LOGSTART2("fileitem::RegisterFileItem", spCustomFileItem->m_sPathRel);
+	LOGSTARTFUNCx(spCustomFileItem->m_sPathRel);
 
 	if (!spCustomFileItem || spCustomFileItem->m_sPathRel.empty())
 		return false;
 
-	Unreg();
+	reset();
 
 	lockguard lockGlobalMap(mapItemsMx);
 
@@ -862,8 +878,8 @@ bool fileItemMgmt::RegisterFileItem(tFileItemPtr spCustomFileItem)
 
 void fileItemMgmt::RegisterFileitemLocalOnly(fileitem* replacement)
 {
-	LOGSTART2("fileItemMgmt::ReplaceWithLocal", replacement);
-	Unreg();
+	LOGSTARTFUNCx(replacement);
+	reset();
 	m_ptr.reset(replacement);
 }
 
@@ -873,10 +889,11 @@ void fileItemMgmt::RegisterFileitemLocalOnly(fileitem* replacement)
 // thread but us is using them.
 time_t fileItemMgmt::BackgroundCleanup()
 {
-	LOGSTART2s("fileItemMgmt::BackgroundCleanup", GetTime());
+	auto now = GetTime();
+	LOGSTARTFUNCsx(now);
 	lockguard lockGlobalMap(mapItemsMx);
 
-	time_t now = GetTime();
+
 	time_t oldestGet = END_OF_TIME;
 	time_t expBefore = now - MAXTEMPDELAY;
 
