@@ -80,14 +80,66 @@ tcpconnect::~tcpconnect()
 	}
 }
 
+class tProtoBlocker
+{
+	atomic_uint v4, v6;
+	// block connection type after that many failed attempts
+	const unsigned threshhold=4;
+    struct timeval block_time = {30,0};
+
+	static void reset_cb(evutil_socket_t, short, void *arg)
+	{
+		auto what = (std::atomic_uint*)arg;
+		what->store(0);
+	};
+public:
+	void report(int proto, unsigned fail_count)
+	{
+		if(0 == fail_count) return;
+
+		atomic_uint* what;
+		switch(proto)
+		{
+		case PF_INET: what=&v4; break;
+		case PF_INET6: what=&v6; break;
+		default: return;
+		}
+		auto before = what->fetch_add(fail_count);
+		if(before < threshhold && (before+fail_count) >= threshhold)
+		{
+			event_base_once(evabase::instance->base, -1, EV_TIMEOUT, reset_cb, &what, &block_time);
+		}
+	}
+	bool can_connect(int proto)
+	{
+		switch (proto)
+		{
+		case PF_INET:
+			return v4 < threshhold;
+		case PF_INET6:
+			return v6 < threshhold;
+		default:
+			return true;
+		}
+
+	}
+} proto_blocker;
+
 inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 {
-	LOGSTART2("tcpconnect::_Connect", "hostname: " << m_sHostName);
+	LOGSTARTFUNCx(m_sHostName, timeout);
 
-	auto dns = CAddrInfo::CachedResolve(m_sHostName, m_sPort, sErrorMsg);
+	auto dns = CAddrInfo::CachedResolve(m_sHostName, m_sPort);
 
-	if(!dns)
+	unsigned timeouts_v4(0), timeouts_v6(0);
+	tDtorEx transfer_timeout_counts([&](){
+		proto_blocker.report(PF_INET, timeouts_v4);
+		proto_blocker.report(PF_INET6, timeouts_v6);
+	});
+
+	if(!dns || dns->HasError())
 	{
+		sErrorMsg = dns->GetError();
 		USRDBG(sErrorMsg);
 		return false; // sErrorMsg got the info already, no other chance to fix it
 	}
@@ -118,6 +170,15 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 		{
 			checkforceclose(fd);
 			tmexp = time_exp;
+
+			// check if temporary blacklisted
+			if(!proto_blocker.can_connect(dns->ai_family))
+			{
+				errno = EAFNOSUPPORT;
+				state = HANDLE_ERROR;
+				return false;
+			}
+
 			fd = ::socket(dns->ai_family, dns->ai_socktype, dns->ai_protocol);
 			if(fd == -1)
 			{
@@ -125,7 +186,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 				return false;
 			}
 			set_connect_sock_flags(fd);
-#if DEBUG
+#ifdef DEBUG
 			log::err(string("Connecting: ") + formatIpPort(dns));
 #endif
 			while (true)
@@ -155,20 +216,43 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 	// pickup the first and/or probably the best errno code which can be reported to user
 	int error_prim = 0;
 
-	auto retGood = [&](int& fd) { std::swap(fd, m_conFd); return true; };
-	auto retError = [&](const std::string &errStr) { sErrorMsg = errStr; return false; };
+	auto retGood = [&](tConData& condata)
+			{
+		LOG(condata.fd);
+		std::swap(condata.fd, m_conFd);
+		// mark this as good protocol, report failure count on other on return later
+		if(condata.dns->ai_family == PF_INET6)
+			timeouts_v6 = 0;
+		else
+			timeouts_v4 = 0;
+		return true;
+	};
+	auto retError = [&](const std::string &errStr) {
+		LOG(errStr);
+		sErrorMsg = errStr;
+		return false;
+	};
 	auto withErrnoError = [&]() { return retError(tErrnoFmter("500 Connection failure: "));	};
 	auto withThisErrno = [&withErrnoError](int myErr) { errno = myErr; return withErrnoError(); };
 
 	// ok, initial condition, one target should be always there, iterator would also hop to the next fallback if allowed
 	if(!prim.dns)
-		return withThisErrno(EAFNOSUPPORT);
+	{
+		dbgline;
+		if(cfg::conprotos[0] == PF_UNSPEC)
+			return retError("523 Target address could not be resolved");
+		else
+			return withThisErrno(EAFNOSUPPORT);
+	}
+
 	if (cfg::fasttimeout > 0)
 	{
+		dbgline;
 		alt.dns = iter.next();
 		alt.state = alt.dns ? NOT_YET : NO_ALTERNATIVES;
 	}
 
+	dbgline;
 	for(auto op_max=0; op_max < 30000; ++op_max) // fail-safe switch, in case of any mistake here
 	{
 		LOG("state a: " << prim.state << ", state b: " << alt.state );
@@ -181,7 +265,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 			break;
 		case ADDR_PICKED:
 			if(prim.init_con(time_start + cfg::nettimeout))
-				return retGood(prim.fd);
+				return retGood(prim);
 			OPTSET(error_prim, errno);
 			__just_fall_through;
 		case SELECT_CONN:
@@ -189,6 +273,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 			if(GetTime() >= prim.tmexp)
 			{
 				OPTSET(error_prim, ELVIS(errno, ETIMEDOUT));
+				++(prim.dns->ai_family == PF_INET ? timeouts_v4 : timeouts_v6);
 				prim.state = HANDLE_ERROR;
 				continue;
 			}
@@ -237,7 +322,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 		case ADDR_PICKED:
 		{
 			if(alt.init_con(GetTime() + cfg::fasttimeout))
-				return retGood(alt.fd);
+				return retGood(alt);
 			continue;
 		}
 		case PICK_ADDR:
@@ -256,6 +341,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 			}
 			if(now >= alt.tmexp)
 			{
+				++(alt.dns->ai_family == PF_INET ? timeouts_v4 : timeouts_v6);
 				alt.state = HANDLE_ERROR;
 				continue;
 			}
@@ -320,7 +406,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 						prim.state = HANDLE_ERROR;
 					}
 					else
-						return retGood(p->fd);
+						return retGood(*p);
 				}
 			}
 		}
@@ -337,7 +423,7 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 
 void tcpconnect::Disconnect()
 {
-	LOGSTART2("tcpconnect::_Disconnect", m_sHostName);
+	LOGSTARTFUNCx(m_sHostName);
 
 #ifdef DEBUG
 	nDisconCount.fetch_add(m_conFd >=0);
@@ -366,8 +452,7 @@ tDlStreamHandle dl_con_factory::CreateConnected(cmstring &sHostname, cmstring &s
 		mstring &sErrOut, bool *pbSecondHand, cfg::tRepoData::IHookHandler *pStateTracker
 		,bool bSsl, int timeout, bool nocache)
 {
-	LOGSTART2s("tcpconnect::CreateConnected", "hostname: " << sHostname << ", port: " << sPort
-			<< (bSsl?" with ssl":" , no ssl"));
+	LOGSTARTFUNCsx(sHostname, sPort, bSsl);
 
 	tDlStreamHandle p;
 #ifndef HAVE_SSL
@@ -448,7 +533,7 @@ void dl_con_factory::RecycleIdleConnection(tDlStreamHandle & handle)
 	if(!handle)
 		return;
 
-	LOGSTART2s("tcpconnect::RecycleIdleConnection", handle->m_sHostName);
+	LOGSTARTFUNCsx(handle->m_sHostName);
 
 	if(handle->m_pStateObserver)
 	{
